package com.sun.jmx.snmp;

import java.util.logging.Level;
import com.sun.jmx.defaults.JmxProperties;

public class SnmpV3Message extends SnmpMsg
{
    public int msgId;
    public int msgMaxSize;
    public byte msgFlags;
    public int msgSecurityModel;
    public byte[] msgSecurityParameters;
    public byte[] contextEngineId;
    public byte[] contextName;
    public byte[] encryptedPdu;
    
    public SnmpV3Message() {
        this.msgId = 0;
        this.msgMaxSize = 0;
        this.msgFlags = 0;
        this.msgSecurityModel = 0;
        this.msgSecurityParameters = null;
        this.contextEngineId = null;
        this.contextName = null;
        this.encryptedPdu = null;
    }
    
    @Override
    public int encodeMessage(byte[] array) throws SnmpTooBigException {
        if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.SNMP_LOGGER.logp(Level.FINER, SnmpV3Message.class.getName(), "encodeMessage", "Can't encode directly V3Message! Need a SecuritySubSystem");
        }
        throw new IllegalArgumentException("Can't encode");
    }
    
    @Override
    public void decodeMessage(byte[] array, int n) throws SnmpStatusException {
        try {
            BerDecoder berDecoder = new BerDecoder(array);
            berDecoder.openSequence();
            this.version = berDecoder.fetchInteger();
            berDecoder.openSequence();
            this.msgId = berDecoder.fetchInteger();
            this.msgMaxSize = berDecoder.fetchInteger();
            this.msgFlags = berDecoder.fetchOctetString()[0];
            this.msgSecurityModel = berDecoder.fetchInteger();
            berDecoder.closeSequence();
            this.msgSecurityParameters = berDecoder.fetchOctetString();
            if ((this.msgFlags & 0x2) == 0x0) {
                berDecoder.openSequence();
                this.contextEngineId = berDecoder.fetchOctetString();
                this.contextName = berDecoder.fetchOctetString();
                this.data = berDecoder.fetchAny();
                this.dataLength = this.data.length;
                berDecoder.closeSequence();
            }
            else {
                this.encryptedPdu = berDecoder.fetchOctetString();
            }
            berDecoder.closeSequence();
        }
        catch (BerException ex) {
            ex.printStackTrace();
            throw new SnmpStatusException("Invalid encoding");
        }
        if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.SNMP_LOGGER.logp(Level.FINER, SnmpV3Message.class.getName(), "decodeMessage", "Unmarshalled message : \n" + "version : " + this.version + "\n" + "msgId : " + this.msgId + "\n" + "msgMaxSize : " + this.msgMaxSize + "\n" + "msgFlags : " + (int)this.msgFlags + "\n" + "msgSecurityModel : " + this.msgSecurityModel + "\n" + "contextEngineId : " + ((this.contextEngineId == null) ? null : SnmpEngineId.createEngineId(this.contextEngineId)) + "\n" + "contextName : " + this.contextName + "\n" + "data : " + this.data + "\n" + "dat len : " + ((this.data == null) ? 0 : this.data.length) + "\n" + "encryptedPdu : " + this.encryptedPdu + "\n");
        }
    }
    
    @Override
    public int getRequestId(byte[] array) throws SnmpStatusException {
        BerDecoder berDecoder;
        int fetchInteger;
        try {
            berDecoder = new BerDecoder(array);
            berDecoder.openSequence();
            berDecoder.fetchInteger();
            berDecoder.openSequence();
            fetchInteger = berDecoder.fetchInteger();
        }
        catch (BerException ex) {
            throw new SnmpStatusException("Invalid encoding");
        }
        try {
            berDecoder.closeSequence();
        }
        catch (BerException ex2) {}
        return fetchInteger;
    }
    
    @Override
    public void encodeSnmpPdu(SnmpPdu snmpPdu, int n) throws SnmpStatusException, SnmpTooBigException {
        SnmpScopedPduPacket snmpScopedPduPacket = (SnmpScopedPduPacket)snmpPdu;
        if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.SNMP_LOGGER.logp(Level.FINER, SnmpV3Message.class.getName(), "encodeSnmpPdu", "PDU to marshall: \n" + "security parameters : " + snmpScopedPduPacket.securityParameters + "\n" + "type : " + snmpScopedPduPacket.type + "\n" + "version : " + snmpScopedPduPacket.version + "\n" + "requestId : " + snmpScopedPduPacket.requestId + "\n" + "msgId : " + snmpScopedPduPacket.msgId + "\n" + "msgMaxSize : " + snmpScopedPduPacket.msgMaxSize + "\n" + "msgFlags : " + (int)snmpScopedPduPacket.msgFlags + "\n" + "msgSecurityModel : " + snmpScopedPduPacket.msgSecurityModel + "\n" + "contextEngineId : " + snmpScopedPduPacket.contextEngineId + "\n" + "contextName : " + snmpScopedPduPacket.contextName + "\n");
        }
        this.version = snmpScopedPduPacket.version;
        this.address = snmpScopedPduPacket.address;
        this.port = snmpScopedPduPacket.port;
        this.msgId = snmpScopedPduPacket.msgId;
        this.msgMaxSize = snmpScopedPduPacket.msgMaxSize;
        this.msgFlags = snmpScopedPduPacket.msgFlags;
        this.msgSecurityModel = snmpScopedPduPacket.msgSecurityModel;
        this.contextEngineId = snmpScopedPduPacket.contextEngineId;
        this.contextName = snmpScopedPduPacket.contextName;
        this.securityParameters = snmpScopedPduPacket.securityParameters;
        this.data = new byte[n];
        try {
            BerEncoder o = new BerEncoder(this.data);
            o.openSequence();
            this.encodeVarBindList(o, snmpScopedPduPacket.varBindList);
            switch (snmpScopedPduPacket.type) {
                case 160:
                case 161:
                case 162:
                case 163:
                case 166:
                case 167:
                case 168: {
                    SnmpPduRequestType snmpPduRequestType = (SnmpPduRequestType)snmpScopedPduPacket;
                    o.putInteger(snmpPduRequestType.getErrorIndex());
                    o.putInteger(snmpPduRequestType.getErrorStatus());
                    o.putInteger(snmpScopedPduPacket.requestId);
                    break;
                }
                case 165: {
                    SnmpPduBulkType snmpPduBulkType = (SnmpPduBulkType)snmpScopedPduPacket;
                    o.putInteger(snmpPduBulkType.getMaxRepetitions());
                    o.putInteger(snmpPduBulkType.getNonRepeaters());
                    o.putInteger(snmpScopedPduPacket.requestId);
                    break;
                }
                default: {
                    throw new SnmpStatusException("Invalid pdu type " + String.valueOf(snmpScopedPduPacket.type));
                }
            }
            o.closeSequence(snmpScopedPduPacket.type);
            this.dataLength = o.trim();
        }
        catch (ArrayIndexOutOfBoundsException ex) {
            throw new SnmpTooBigException();
        }
    }
    
    @Override
    public SnmpPdu decodeSnmpPdu() throws SnmpStatusException {
        BerDecoder berDecoder = new BerDecoder(this.data);
        SnmpScopedPduRequest snmpScopedPduRequest2;
        try {
            int tag = berDecoder.getTag();
            berDecoder.openSequence(tag);
            switch (tag) {
                case 160:
                case 161:
                case 162:
                case 163:
                case 166:
                case 167:
                case 168: {
                    SnmpScopedPduRequest snmpScopedPduRequest = new SnmpScopedPduRequest();
                    snmpScopedPduRequest.requestId = berDecoder.fetchInteger();
                    snmpScopedPduRequest.setErrorStatus(berDecoder.fetchInteger());
                    snmpScopedPduRequest.setErrorIndex(berDecoder.fetchInteger());
                    snmpScopedPduRequest2 = snmpScopedPduRequest;
                    break;
                }
                case 165: {
                    SnmpScopedPduBulk snmpScopedPduBulk = new SnmpScopedPduBulk();
                    snmpScopedPduBulk.requestId = berDecoder.fetchInteger();
                    snmpScopedPduBulk.setNonRepeaters(berDecoder.fetchInteger());
                    snmpScopedPduBulk.setMaxRepetitions(berDecoder.fetchInteger());
                    snmpScopedPduRequest2 = snmpScopedPduBulk;
                    break;
                }
                default: {
                    throw new SnmpStatusException(9);
                }
            }
            snmpScopedPduRequest2.type = tag;
            snmpScopedPduRequest2.varBindList = this.decodeVarBindList(berDecoder);
            berDecoder.closeSequence();
        }
        catch (BerException thrown) {
            if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINEST)) {
                JmxProperties.SNMP_LOGGER.logp(Level.FINEST, SnmpV3Message.class.getName(), "decodeSnmpPdu", "BerException", thrown);
            }
            throw new SnmpStatusException(9);
        }
        snmpScopedPduRequest2.address = this.address;
        snmpScopedPduRequest2.port = this.port;
        snmpScopedPduRequest2.msgFlags = this.msgFlags;
        snmpScopedPduRequest2.version = this.version;
        snmpScopedPduRequest2.msgId = this.msgId;
        snmpScopedPduRequest2.msgMaxSize = this.msgMaxSize;
        snmpScopedPduRequest2.msgSecurityModel = this.msgSecurityModel;
        snmpScopedPduRequest2.contextEngineId = this.contextEngineId;
        snmpScopedPduRequest2.contextName = this.contextName;
        snmpScopedPduRequest2.securityParameters = this.securityParameters;
        if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.SNMP_LOGGER.logp(Level.FINER, SnmpV3Message.class.getName(), "decodeSnmpPdu", "Unmarshalled PDU : \n" + "type : " + snmpScopedPduRequest2.type + "\n" + "version : " + snmpScopedPduRequest2.version + "\n" + "requestId : " + snmpScopedPduRequest2.requestId + "\n" + "msgId : " + snmpScopedPduRequest2.msgId + "\n" + "msgMaxSize : " + snmpScopedPduRequest2.msgMaxSize + "\n" + "msgFlags : " + (int)snmpScopedPduRequest2.msgFlags + "\n" + "msgSecurityModel : " + snmpScopedPduRequest2.msgSecurityModel + "\n" + "contextEngineId : " + snmpScopedPduRequest2.contextEngineId + "\n" + "contextName : " + snmpScopedPduRequest2.contextName + "\n");
            return snmpScopedPduRequest2;
        }
        return snmpScopedPduRequest2;
    }
    
    @Override
    public String printMessage() {
        StringBuffer sb = new StringBuffer();
        sb.append("msgId : " + this.msgId + "\n");
        sb.append("msgMaxSize : " + this.msgMaxSize + "\n");
        sb.append("msgFlags : " + (int)this.msgFlags + "\n");
        sb.append("msgSecurityModel : " + this.msgSecurityModel + "\n");
        if (this.contextEngineId == null) {
            sb.append("contextEngineId : null");
        }
        else {
            sb.append("contextEngineId : {\n");
            sb.append(SnmpMsg.dumpHexBuffer(this.contextEngineId, 0, this.contextEngineId.length));
            sb.append("\n}\n");
        }
        if (this.contextName == null) {
            sb.append("contextName : null");
        }
        else {
            sb.append("contextName : {\n");
            sb.append(SnmpMsg.dumpHexBuffer(this.contextName, 0, this.contextName.length));
            sb.append("\n}\n");
        }
        return sb.append(super.printMessage()).toString();
    }
}
